@@ -1,4 +1,5 @@
 import os
+import sys
 import numpy as np
 import glob
 from sklearn.model_selection import StratifiedShuffleSplit
@@ -10,7 +11,9 @@ import wave
 import contextlib
 from collections import Counter
 
-def load(folders=None, test_val=[0.2, 0.2], test=True, validation=True):
+
+def load(folders=None, test_val=[0.2, 0.2],
+         test=True, validation=True, class_mapping=None):
     """Loads a dataset from some folders.
 
     Arguments
@@ -43,6 +46,8 @@ def load(folders=None, test_val=[0.2, 0.2], test=True, validation=True):
     filenames = []
     labels = []
 
+    folders = sorted(folders)
+
     # Match filenames with labels
     for folder in folders:
         for f in glob.iglob(os.path.join(folder, '*.wav')):
@@ -51,9 +56,30 @@ def load(folders=None, test_val=[0.2, 0.2], test=True, validation=True):
 
     # Convert labels to int
     folder2idx, idx2folder = folders_mapping(folders=folders)
+    if class_mapping:
+        for k, v in idx2folder.items():
+            unseen_class = True
+            for k_2, v_2 in class_mapping.items():
+                if v_2 in v:
+                    idx2folder[k_2] = v
+                    unseen_class = False
+            
+            if unseen_class:
+                 sys.exit("Error: You provided a class name that was not seen during training. "
+                          "Please use the exact same class names that were used for training.")
+                
+        folder2idx = {v: k for k, v in idx2folder.items()}
 
     labels = list(map(lambda x: folder2idx[x], labels))
-    class_mapping = {name: idx2folder[name].split("/")[-1] for name in idx2folder}
+
+    class_mapping = {}
+    for name in idx2folder:
+        directories = idx2folder[name].split("/")
+        if directories[-1] == "":
+            class_mapping[name] = directories[-2]
+        else:
+            class_mapping[name] = directories[-1]
+
     # Split
     if test is False and validation is False:
         # Use this data only to train
@@ -124,8 +150,14 @@ def compute_max_seq_len(reload=False, X=None, folders=None):
                           (config.WINDOW_LENGTH - config.HOP_LENGTH)) / \
                          (config.HOP_LENGTH) + 1)
             lengths.append(length)
-    max_seq = np.max(lengths)
+    # instead of computing the overall maximum we use the 90% percentile
+    max_seq = int(np.max(lengths))
     print(f"Max sequence length in dataset: {max_seq}")
+    mean_seq = int(np.mean(lengths))
+    print(f"Mean sequence length in dataset: {mean_seq}")
+    std_seq = int(np.std(lengths))
+    print(f"Std of sequence lengths in dataset: {std_seq}")
+
     return max_seq
 
 
